@@ -36,10 +36,11 @@ namespace Gdk {
 		static extern int gtksharp_gdk_event_expose_get_count (IntPtr evt);
 
 		public EventExpose (IntPtr raw) : base (raw) {} 
+		public EventExpose (IntPtr raw, bool owned) : base (raw, owned) {}
 
 		public Rectangle Area {
 			get {
-				return (Gdk.Rectangle) Marshal.PtrToStructure (gtksharp_gdk_event_expose_get_area (Handle), typeof (Gdk.Rectangle));
+				return Marshal.PtrToStructure<Gdk.Rectangle> (gtksharp_gdk_event_expose_get_area (Handle));
 			}
 		}
 
