@@ -26,13 +26,14 @@ namespace Gdk {
 
 	public class EventProximity : Event {
 
-		[DllImport("gdksharpglue-2")]
+		[DllImport("gdksharpglue-2", CallingConvention=CallingConvention.Cdecl)]
 		static extern uint gtksharp_gdk_event_proximity_get_time (IntPtr evt);
 
-		[DllImport("gdksharpglue-2")]
+		[DllImport("gdksharpglue-2", CallingConvention=CallingConvention.Cdecl)]
 		static extern IntPtr gtksharp_gdk_event_proximity_get_device (IntPtr evt);
 
-		public EventProximity (IntPtr raw) : base (raw) {} 
+		public EventProximity (IntPtr raw) : base (raw) { }
+		public EventProximity (IntPtr raw, bool owned) : base (raw, owned) { }
 
 		public Device Device {
 			get {
