@@ -26,7 +26,7 @@ namespace GtkSharp.Generation {
 
 	public class ByRefGen : SimpleBase {
 		
-		public ByRefGen (string ctype, string type) : base (ctype, type) {}
+		public ByRefGen (string ctype, string type) : base (ctype, type, type + ".Empty") {}
 		
 		public override string MarshalType {
 			get {
@@ -34,27 +34,11 @@ namespace GtkSharp.Generation {
 			}
 		}
 
-		public override string MarshalReturnType {
+		public override string MarshalCallbackType {
 			get {
-				return QualifiedName;
+				return "ref " + QualifiedName;
 			}
 		}
-
-		public override string ToNativeReturnType {
-			get {
-				return QualifiedName;
-			}
-		}
-
-		public override string CallByName (string var_name)
-		{
-			return "ref " + var_name;
-		}
-		
-		public override string ToNativeReturn(string var)
-		{
-			return var;
-		}
 	}
 }
 
