@@ -111,6 +111,8 @@ namespace GtkSharp.Generation {
 
 				if (needs_chaining) {
 					sw.WriteLine ("\t\t\tif (GetType () != typeof (" + name + ")) {");
+					if (gen_info.AssemblyName == "gtk-sharp")
+						sw.WriteLine ("\t\t\t\tGtk.Application.AssertMainThread();");
 					
 					if (Parameters.Count == 0) {
 						sw.WriteLine ("\t\t\t\tCreateNativeObject (Array.Empty<IntPtr> (), Array.Empty<GLib.Value> (), 0);");
@@ -173,7 +175,10 @@ namespace GtkSharp.Generation {
 					sw.WriteLine ("\t\t\t}");
 				}
 	
-				Body.Initialize(gen_info, false, false, "", false); 
+				Body.Initialize(gen_info, false, false, "", false);
+				if (container_type is ObjectGen) {
+					sw.WriteLine ("\t\t\towned = true;");
+				}
 				sw.WriteLine("\t\t\t{0} = {1}({2});", container_type.AssignToName, CName, Body.GetCallString (false));
 				Body.Finish (sw, "");
 				Body.HandleException (sw, "");
