@@ -139,10 +139,10 @@ namespace GtkSharp.Generation {
 		{
 			sw.WriteLine ("\t\tstatic void Initialize (IntPtr ifaceptr, IntPtr data)");
 			sw.WriteLine ("\t\t{");
-			sw.WriteLine ("\t\t\t" + IfaceName + " native_iface = (" + IfaceName + ") Marshal.PtrToStructure (ifaceptr, typeof (" + IfaceName + "));");
+			sw.WriteLine ("\t\t\t" + IfaceName + " native_iface = Marshal.PtrToStructure<" + IfaceName + "> (ifaceptr);");
 			foreach (VirtualMethod vm in vms)
 				sw.WriteLine ("\t\t\tnative_iface." + vm.CName + " = iface." + vm.CName + ";");
-			sw.WriteLine ("\t\t\tMarshal.StructureToPtr (native_iface, ifaceptr, false);");
+			sw.WriteLine ("\t\t\tMarshal.StructureToPtr<" + IfaceName + "> (native_iface, ifaceptr, false);");
 			sw.WriteLine ("\t\t\tGCHandle gch = (GCHandle) data;");
 			sw.WriteLine ("\t\t\tgch.Free ();");
 			sw.WriteLine ("\t\t}");
@@ -170,6 +170,8 @@ namespace GtkSharp.Generation {
 			sw.WriteLine ();
 			sw.WriteLine ("\t\tpublic {0}Adapter ({0}Implementor implementor)", Name);
 			sw.WriteLine ("\t\t{");
+			sw.WriteLine ("\t\t\tif (implementor == null)");
+			sw.WriteLine ("\t\t\t\tthrow new ArgumentNullException (\"implementor\");");
 			sw.WriteLine ("\t\t\tthis.implementor = implementor;");
 			sw.WriteLine ("\t\t}");
 			sw.WriteLine ();
@@ -279,15 +281,7 @@ namespace GtkSharp.Generation {
 
 			sw.WriteLine ("#endregion");
 
-			string custom = Path.Combine (gen_info.CustomDir, Name + "Adapter.custom");
-			if (File.Exists (custom)) {
-				sw.WriteLine ("#region Customized extensions");
-				sw.WriteLine ("#line 1 \"" + Name + "Adapter.custom\"");
-				using (StreamReader sr = new StreamReader(new FileStream (custom, FileMode.Open, FileAccess.Read)))
-					sw.WriteLine (sr.ReadToEnd ());
-				
-				sw.WriteLine ("#endregion");
-			}
+			AppendCustom (sw, gen_info.CustomDir, Name + "Adapter");
 
 			sw.WriteLine ("\t}");
 			sw.WriteLine ("}");
@@ -295,14 +289,16 @@ namespace GtkSharp.Generation {
 			gen_info.Writer = null;
 		}
 
-		void GenerateImplementorIface (StreamWriter sw)
+		void GenerateImplementorIface (GenerationInfo gen_info)
 		{
+			StreamWriter sw = gen_info.Writer;
 			if (IsConsumeOnly)
 				return;
 
 			sw.WriteLine ();
 			sw.WriteLine ("\t[GLib.GInterface (typeof (" + Name + "Adapter))]");
-			sw.WriteLine ("\tpublic interface " + Name + "Implementor : GLib.IWrapper {");
+			string access = IsInternal ? "internal" : "public";
+			sw.WriteLine ("\t" + access + " interface " + Name + "Implementor : GLib.IWrapper {");
 			sw.WriteLine ();
 			Hashtable vm_table = new Hashtable ();
 			foreach (VirtualMethod vm in vms)
@@ -330,6 +326,9 @@ namespace GtkSharp.Generation {
 					vm_table.Remove (vm.Name);
 				}
 			}
+
+			AppendCustom (sw, gen_info.CustomDir, Name + "Implementor");
+
 			sw.WriteLine ("\t}");
 		}
 
@@ -343,7 +342,8 @@ namespace GtkSharp.Generation {
 			sw.WriteLine ("\tusing System;");
 			sw.WriteLine ();
 			sw.WriteLine ("#region Autogenerated code");
-			sw.WriteLine ("\tpublic interface " + Name + " : GLib.IWrapper {");
+			string access = IsInternal ? "internal" : "public";
+			sw.WriteLine ("\t" + access + " interface " + Name + " : GLib.IWrapper {");
 			sw.WriteLine ();
 			
 			foreach (Signal sig in sigs.Values) {
@@ -363,7 +363,7 @@ namespace GtkSharp.Generation {
 			AppendCustom (sw, gen_info.CustomDir);
 
 			sw.WriteLine ("\t}");
-			GenerateImplementorIface (sw);
+			GenerateImplementorIface (gen_info);
 			sw.WriteLine ("#endregion");
 			sw.WriteLine ("}");
 			sw.Close ();
