@@ -29,24 +29,55 @@ namespace GtkSharp.Generation {
 		
 		ArrayList parms = new ArrayList ();
 		ArrayList special = new ArrayList ();
+		string error_param = null;
+		string user_data_param = null;
+		string destroy_param = null;
 
-		public ManagedCallString (Parameters parms)
+		public ManagedCallString (Parameters parms, bool drop_first)
 		{
-			for (int i = 1; i < parms.Count; i ++) {
+			for (int i = drop_first ? 1 : 0; i < parms.Count; i ++) {
 				Parameter p = parms [i];
-				if (p.IsLength && parms [i-1].IsString)
+				if (p.IsLength && i > 0 && parms [i-1].IsString) 
 					continue;
+				else if (p.Scope == "notified") {
+					user_data_param = parms[i+1].Name;
+					destroy_param = parms[i+2].Name;
+					i += 2;
+				} else if (p.IsUserData && parms.IsHidden (p)) {
+					user_data_param = p.Name;
+					continue;
+				} else if (p is ErrorParameter) {
+					error_param = p.Name;
+					continue;
+				}
 				this.parms.Add (p);
 
-				IGeneratable igen = p.Generatable;
-				if ((igen is StructGen || p.PassAs != "") &&
-				    (p.Name != p.FromNative (p.Name)))
+				if (p.PassAs != String.Empty && (p.Name != p.FromNative (p.Name)))
+					this.special.Add (true);
+				else if (p.Generatable is CallbackGen)
 					this.special.Add (true);
 				else
 					this.special.Add (false);
 			}
 		}
 
+		public bool HasOutParam {
+			get {
+				foreach (Parameter p in parms) {
+					if (p.PassAs == "out")
+						return true;
+				}
+				return false;
+			}
+		}
+
+		public string Unconditional (string indent) {
+			string ret = "";
+			if (error_param != null)
+				ret = indent + error_param + " = IntPtr.Zero;\n";
+			return ret;
+		}
+
 		public string Setup (string indent)
 		{
 			string ret = "";
@@ -58,10 +89,20 @@ namespace GtkSharp.Generation {
 				Parameter p = parms [i] as Parameter;
 				IGeneratable igen = p.Generatable;
 
-				ret += indent + igen.QualifiedName + " my" + p.Name;
-				if (igen is StructGen || p.PassAs == "ref")
-					ret += " = " + p.FromNative (p.Name);
-				ret += ";\n";
+				if (igen is CallbackGen) {
+					if (user_data_param == null)
+						ret += indent + String.Format ("{0} {1}_invoker = new {0} ({1});\n", (igen as CallbackGen).InvokerName, p.Name);
+					else if (destroy_param == null)
+						ret += indent + String.Format ("{0} {1}_invoker = new {0} ({1}, {2});\n", (igen as CallbackGen).InvokerName, p.Name, user_data_param);
+					else
+						ret += indent + String.Format ("{0} {1}_invoker = new {0} ({1}, {2}, {3});\n", (igen as CallbackGen).InvokerName, p.Name, user_data_param, destroy_param);
+				} else {
+					ret += indent + igen.QualifiedName + " my" + p.Name;
+					if (p.PassAs == "ref") {
+						ret += " = " + p.FromNative (p.Name);
+					}
+					ret += ";\n";
+				}
 			}
 
 			return ret;
@@ -77,7 +118,11 @@ namespace GtkSharp.Generation {
 			for (int i = 0; i < parms.Count; i ++) {
 				Parameter p = parms [i] as Parameter;
 				result [i] = p.PassAs == "" ? "" : p.PassAs + " ";
-				result [i] += ((bool)special[i]) ? "my" + p.Name : p.FromNative (p.Name);
+
+				if (p.Generatable is CallbackGen)
+					result [i] += p.Name + "_invoker.Handler";
+				else
+					result [i] += ((bool)special[i]) ? "my" + p.Name : p.FromNative (p.Name);
 			}
 
 			return String.Join (", ", result);
@@ -94,9 +139,16 @@ namespace GtkSharp.Generation {
 				Parameter p = parms [i] as Parameter;
 				IGeneratable igen = p.Generatable;
 
-				if (igen is StructBase)
-					ret += indent + String.Format ("if ({0} != IntPtr.Zero) System.Runtime.InteropServices.Marshal.StructureToPtr (my{0}, {0}, false);\n", p.Name);
-				else
+				if (igen is CallbackGen)
+					continue;
+				if (igen is StructBase) {
+					if (SymbolTable.Table.IsBlittable (igen)) {
+						ret += indent + string.Format ("unsafe {{ if ({1} != IntPtr.Zero) *({0}*){1} = my{1}; }}\n", p.CSType, p.Name);
+					} else {
+						ret += indent + String.Format ("if ({0} != IntPtr.Zero) System.Runtime.InteropServices.Marshal.StructureToPtr (my{0}, {0}, false);\n", p.Name);
+					}
+				}
+				else if (!(igen is ByRefGen))
 					ret += indent + p.Name + " = " + igen.ToNativeReturn ("my" + p.Name) + ";\n";
 			}
 
