@@ -168,6 +168,19 @@ namespace GtkSharp.Generation {
 			}
 		}
 
+		public virtual string MarshalCallbackType {
+			get {
+				string type = SymbolTable.Table.GetMarshalCallbackType (elem.GetAttribute ("type"));
+				if (type == "void" || Generatable is IManualMarshaler)
+					type = "IntPtr";
+				if (IsArray) {
+					type += "[]";
+					type = type.Replace ("ref ", "");
+				}
+				return type;
+			}
+		}
+
 		public string Name {
 			get {
 				return SymbolTable.Table.MangleName (elem.GetAttribute("name"));
@@ -180,6 +193,15 @@ namespace GtkSharp.Generation {
 			}
 		}
 
+		public virtual string NativeCallbackSignature {
+			get {
+				string sig = MarshalCallbackType + " " + Name;
+				if (PassAs != String.Empty)
+					sig = PassAs + " " + sig;
+				return sig;
+			}
+		}
+
 		public virtual string NativeSignature {
 			get {
 				string sig = MarshalType + " " + Name;
@@ -245,6 +267,12 @@ namespace GtkSharp.Generation {
 			}
 		}
 
+		public virtual string [] PrepareCallback {
+			get {
+				return Prepare;
+			}
+		}
+
 		public virtual string CallString {
 			get {
 				string call_parm;
@@ -266,6 +294,12 @@ namespace GtkSharp.Generation {
 			}
 		}
 
+		public virtual string CallStringCallback {
+			get {
+				return CallString;
+			}
+		}
+
 		public virtual string[] Finish {
 			get {
 				IGeneratable gen = Generatable;
@@ -277,12 +311,23 @@ namespace GtkSharp.Generation {
 					if (PassAs != "out")
 						result [i] = (gen as IManualMarshaler).ReleaseNative ("native_" + CallName) + ";";
 					return result;
-				} else if (PassAs != String.Empty && MarshalType != CSType)
+				} else if (PassAs != String.Empty && MarshalType != CSType) {
+					var handle = gen as HandleBase;
+					if (handle != null) {
+						return new string [] { CallName + " = " + handle.FromNative ("native_" + CallName, Owned) + ";" };
+					}
 					return new string [] { CallName + " = " + gen.FromNative ("native_" + CallName) + ";" };
+				}
 				return new string [0];
 			}
 		}
 
+		public virtual string [] FinishCallback {
+			get {
+				return Finish;
+			}
+		}
+
 		public string FromNative (string var)
 		{
 			if (Generatable == null)
@@ -340,16 +385,20 @@ namespace GtkSharp.Generation {
 
 				ArrayList result = new ArrayList ();
 				result.Add (String.Format ("int cnt_{0} = {0} == null ? 0 : {0}.Length;", CallName));
-				result.Add (String.Format ("{0}[] native_{1} = new {0} [cnt_{1}" + (NullTerminated ? " + 1" : "") + "];", MarshalType.TrimEnd('[', ']'), CallName));
-				result.Add (String.Format ("for (int i = 0; i < cnt_{0}; i++)", CallName));
+				result.Add (String.Format ("{0}[] native_{1} = new {0} [cnt_{1}" + (NullTerminated ? " + 1" : "") + "];", MarshalType.TrimEnd ('[', ']'), CallName));
 				IGeneratable gen = Generatable;
+				result.Add (String.Format ("for (int i = 0; i < cnt_{0}; i++) {{", CallName));
+				string marshalRes;
 				if (gen is IManualMarshaler)
-					result.Add (String.Format ("\tnative_{0} [i] = {1};", CallName, (gen as IManualMarshaler).AllocNative (CallName + "[i]")));
+					marshalRes = ((IManualMarshaler)gen).AllocNative (CallName + "[i]");
 				else
-					result.Add (String.Format ("\tnative_{0} [i] = {1};", CallName, gen.CallByName (CallName + "[i]")));
+					marshalRes = gen.CallByName (CallName + "[i]");
 
+				result.Add (String.Format ("\tnative_{0} [i] = {1};", CallName, marshalRes));
+				result.Add ("}");
 				if (NullTerminated)
 					result.Add (String.Format ("native_{0} [cnt_{0}] = IntPtr.Zero;", CallName));
+				
 				return (string[]) result.ToArray (typeof (string));
 			}
 		}
@@ -368,17 +417,24 @@ namespace GtkSharp.Generation {
 				if (CSType == MarshalType)
 					return new string [0];
 
+				var result = new System.Collections.Generic.List<string> ();
 				IGeneratable gen = Generatable;
-				if (gen is IManualMarshaler) {
-					string [] result = new string [4];
-					result [0] = "for (int i = 0; i < native_" + CallName + ".Length" + (NullTerminated ? " - 1" : "") + "; i++) {";
-					result [1] = "\t" + CallName + " [i] = " + Generatable.FromNative ("native_" + CallName + "[i]") + ";";
-					result [2] = "\t" + (gen as IManualMarshaler).ReleaseNative ("native_" + CallName + "[i]") + ";";
-					result [3] = "}";
-					return result;
+				if (gen is IManualMarshaler || (gen is MarshalGen && ((MarshalGen)gen).FreeAfterUse)) {
+					result.Add ("for (int i = 0; i < cnt_" + CallName + "; i++) {");
+					string nativeVariable = "native_" + CallName + "[i]";
+					if (gen is IManualMarshaler) {
+						// All of these are in setter methods, it's mostly fire and forget so there's no need to marshal back the return value.
+						//result.Add ("\t" + CallName + "[i] = " + gen.FromNative (nativeVariable) + ";");
+						result.Add ("\t" + (gen as IManualMarshaler).ReleaseNative (nativeVariable) + ";");
+					} else {
+						if (gen is MarshalGen && ((MarshalGen)gen).FreeAfterUse) {
+							// Don't use the FromNative return as it'll try and reconstruct the value, which is useless overhead.
+							result.Add ("\tGLib.Marshaller.Free (" + nativeVariable + ");");
+						}
+					}
+					result.Add ("}");
 				}
-
-				return new string [0];
+				return result.ToArray ();
 			}
 		}
 	}
@@ -437,6 +493,12 @@ namespace GtkSharp.Generation {
 			}
 		}
 
+		public override string NativeCallbackSignature {
+			get {
+				return NativeSignature;
+			}
+		}
+
 		public override string NativeSignature {
 			get {
 				if (invert)
@@ -466,6 +528,12 @@ namespace GtkSharp.Generation {
 		public StructParameter (XmlElement elem) : base (elem) {}
 
 		public override string MarshalType {
+			get {
+				return "ref " + Generatable.QualifiedName;
+			}
+		}
+
+		public override string MarshalCallbackType {
 			get {
 				return "IntPtr";
 			}
@@ -474,13 +542,27 @@ namespace GtkSharp.Generation {
 		public override string[] Prepare {
 			get {
 				if (PassAs == "out")
-					return new string [] { "IntPtr native_" + CallName + " = Marshal.AllocHGlobal (Marshal.SizeOf (typeof (" + Generatable.QualifiedName + ")));"};
+					return new string [] { CallName + " = new " + Generatable.QualifiedName + "();"};
+				return new string [0];
+			}
+		}
+
+		public override string [] PrepareCallback {
+			get {
+				if (PassAs == "out")
+					return new string [] { "IntPtr native_" + CallName + " = Marshal.AllocHGlobal (Marshal.SizeOf (typeof (" + Generatable.QualifiedName + ")));" };
 				else
-					return new string [] { "IntPtr native_" + CallName + " = " + (Generatable as IManualMarshaler).AllocNative (CallName) + ";"};
+					return new string [] { "IntPtr native_" + CallName + " = " + (Generatable as IManualMarshaler).AllocNative (CallName) + ";" };
 			}
 		}
 
 		public override string CallString {
+			get {
+				return "ref " + CallName;
+			}
+		}
+
+		public override string CallStringCallback {
 			get {
 				return "native_" + CallName;
 			}
@@ -488,16 +570,73 @@ namespace GtkSharp.Generation {
 
 		public override string[] Finish {
 			get {
-				string[] result = new string [2];
-				result [0] = CallName + " = " + FromNative ("native_" + CallName) + ";";
-				result [1] = (Generatable as IManualMarshaler).ReleaseNative ("native_" + CallName) + ";";
+				return new string [0];
+			}
+		}
+
+		public override string [] FinishCallback {
+			get {
+				string [] result = new string [PassAs == string.Empty ? 1 : 2];
+				int i = 0;
+				if (PassAs != string.Empty) {
+					result [i++] = CallName + " = " + FromNative ("native_" + CallName) + ";";
+				}
+				result [i++] = (Generatable as IManualMarshaler).ReleaseNative ("native_" + CallName) + ";";
 				return result;
 			}
 		}
 
+		public override string NativeCallbackSignature {
+			get {
+				return "IntPtr " + CallName;
+			}
+		}
+
 		public override string NativeSignature {
 			get {
-				return "IntPtr " + CallName;
+				return "ref " + Generatable.QualifiedName + " " + CallName;
+			}
+		}
+	}
+
+	public class ByRefParameter : Parameter
+	{
+
+		public ByRefParameter (XmlElement elem) : base (elem) { }
+
+		public override string MarshalType {
+			get {
+				return "ref " + CallName;
+			}
+		}
+
+		public override string [] Prepare {
+			get {
+				return new string [0];
+			}
+		}
+
+		public override string CallString {
+			get {
+				return "ref " + CallName;
+			}
+		}
+
+		public override string [] Finish {
+			get {
+				return new string [0];
+			}
+		}
+
+		public override string NativeCallbackSignature {
+			get {
+				return NativeSignature;
+			}
+		}
+
+		public override string NativeSignature {
+			get {
+				return "ref " + CSType + " " + CallName;
 			}
 		}
 	}
@@ -554,6 +693,8 @@ namespace GtkSharp.Generation {
 			if (HasCB || HideData) {
 				if (p.IsUserData && (idx == Count - 1))
                                         return true;
+				if (p.IsUserData && (idx == Count - 2) && this [Count - 1] is ErrorParameter)
+                                        return true;
 				if (p.IsUserData && idx > 0 &&
 				    this [idx - 1].Generatable is CallbackGen)
 					return true;
@@ -658,8 +799,10 @@ namespace GtkSharp.Generation {
 					}
 				} else if (p.CType == "GError**")
 					p = new ErrorParameter (parm);
-				else if (gen is StructBase || gen is ByRefGen) {
+				else if (gen is StructBase) {
 					p = new StructParameter (parm);
+				} else if (gen is ByRefGen) {
+					p = new ByRefParameter (parm);
 				} else if (gen is CallbackGen) {
 					has_cb = true;
 				}
@@ -709,6 +852,19 @@ namespace GtkSharp.Generation {
 			}
 		}
 
+		public string CallbackImportSignature {
+			get {
+				if (Count == 0)
+					return String.Empty;
+
+				string [] result = new string [Count];
+				for (int i = 0; i < Count; i++)
+					result [i] = this [i].NativeCallbackSignature;
+
+				return String.Join (", ", result);
+			}
+		}
+
 		public string ImportSignature {
 			get {
 				if (Count == 0)
