@@ -31,6 +31,7 @@ namespace GtkSharp.Generation {
 		bool is_array;
 		bool elements_owned;
 		bool owned;
+		bool needs_ref;
 		string ctype = String.Empty;
 		string element_ctype = String.Empty;
 
@@ -43,6 +44,7 @@ namespace GtkSharp.Generation {
 				owned = elem.GetAttribute ("owned") == "true";
 				ctype = elem.GetAttribute("type");
 				element_ctype = elem.GetAttribute ("element_type");
+				needs_ref = elem.GetAttribute ("needs_ref") == "true";
 			}
 		}
 
@@ -134,6 +136,14 @@ namespace GtkSharp.Generation {
 			else
 				return IGen.FromNativeReturn (var);
 		}
+
+		public string PostFromNative (string var)
+		{
+			if (!needs_ref)
+				return string.Empty;
+
+			return "g_object_ref (" + var + ");";
+		}
 			
 		public string ToNative (string var)
 		{
