@@ -6,7 +6,7 @@
 // Copyright (c) 2005 Novell, Inc.
 //
 // This program is free software; you can redistribute it and/or
-// modify it under the terms of version 2 of the Lesser GNU General 
+// modify it under the terms of version 2 of the Lesser GNU General
 // Public License as published by the Free Software Foundation.
 //
 // This program is distributed in the hope that it will be useful,
@@ -26,40 +26,40 @@ namespace GLib {
 	using System.Collections;
 	using System.Runtime.InteropServices;
 
+	public delegate void ListElementFree (IntPtr ptr);
 	public abstract class ListBase : IDisposable, ICollection, GLib.IWrapper, ICloneable {
 
 		private IntPtr list_ptr = IntPtr.Zero;
-		private int length = -1;
 		private bool managed = false;
 		internal bool elements_owned = false;
 		protected System.Type element_type = null;
+		private ListElementFree free_func;
 
 		abstract internal IntPtr NthData (uint index);
-		abstract internal IntPtr GetData (IntPtr current);
-		abstract internal IntPtr Next (IntPtr current);
 		abstract internal int Length (IntPtr list);
 		abstract internal void Free (IntPtr list);
 		abstract internal IntPtr Append (IntPtr current, IntPtr raw);
 		abstract internal IntPtr Prepend (IntPtr current, IntPtr raw);
 
-		internal ListBase (IntPtr list, System.Type element_type, bool owned, bool elements_owned)
+		internal ListBase (IntPtr list, System.Type element_type, bool owned, bool elements_owned, ListElementFree free_func)
 		{
 			list_ptr = list;
 			this.element_type = element_type;
 			managed = owned;
 			this.elements_owned = elements_owned;
+			this.free_func = free_func;
 		}
-		
+
 		~ListBase ()
 		{
 			Dispose (false);
 		}
-		
+
 		[Obsolete ("Replaced by owned parameter on ctor.")]
 		public bool Managed {
 			set { managed = value; }
 		}
-		
+
 		public IntPtr Handle {
 			get {
 				return list_ptr;
@@ -89,14 +89,12 @@ namespace GLib {
 		// ICollection
 		public int Count {
 			get {
-				if (length == -1)
-					length = Length (list_ptr);
-				return length;
+				return Length (list_ptr);
 			}
 		}
 
-		public object this [int index] { 
-			get { 
+		public object this [int index] {
+			get {
 				IntPtr data = NthData ((uint) index);
 				object ret = null;
 				ret = DataMarshal (data);
@@ -115,12 +113,24 @@ namespace GLib {
 
 		public void CopyTo (Array array, int index)
 		{
-			object[] orig = new object[Count];
+			CopyTo (array, index, Count);
+		}
+
+		internal void CopyTo (Array array, int index, int count)
+		{
+			object[] orig = new object[count];
 			int i = 0;
 			foreach (object o in this)
 				orig [i++] = o;
-			
-			orig.CopyTo (array, index); 
+
+			orig.CopyTo (array, index);
+		}
+
+		public void CopyTo<T> (T[] array, int index)
+		{
+			int i = index;
+			foreach (T o in this)
+				array [i++] = o;
 		}
 
 		public class FilenameString {
@@ -151,7 +161,7 @@ namespace GLib {
 			return IntPtr.Zero;
 		}
 
-		internal object DataMarshal (IntPtr data) 
+		internal object DataMarshal (IntPtr data)
 		{
 			object ret = null;
 			if (element_type != null) {
@@ -169,7 +179,11 @@ namespace GLib {
 					ret = (int) data;
 				else if (element_type.IsValueType)
 					ret = Marshal.PtrToStructure (data, element_type);
-				else
+				else if (element_type.IsInterface) {
+					Type adapter_type = element_type.Assembly.GetType (element_type.FullName + "Adapter");
+					System.Reflection.MethodInfo method = adapter_type.GetMethod ("GetObject", new Type[] {typeof(IntPtr), typeof(bool)});
+					ret = method.Invoke (null, new object[] {data, false});
+				} else
 					ret = Activator.CreateInstance (element_type, new object[] {data});
 
 			} else if (Object.IsObject (data))
@@ -178,27 +192,50 @@ namespace GLib {
 			return ret;
 		}
 
-		[DllImport ("libglib-2.0-0.dll")]
-		static extern void g_free (IntPtr item);
-
-		[DllImport ("libglib-2.0-0.dll")]
+		[DllImport ("libgobject-2.0-0.dll", CallingConvention=CallingConvention.Cdecl)]
 		static extern void g_object_unref (IntPtr item);
 
 		public void Empty ()
 		{
-			if (elements_owned)
-				for (uint i = 0; i < Count; i++)
-					if (typeof (GLib.Object).IsAssignableFrom (element_type))
-						g_object_unref (NthData (i));
-					else if (typeof (GLib.Opaque).IsAssignableFrom (element_type))
-						GLib.Opaque.GetOpaque (NthData (i), element_type, true).Dispose ();
-					else 
-						g_free (NthData (i));
+			if (elements_owned) {
+				var current = list_ptr;
+				if (typeof (GLib.Object).IsAssignableFrom (element_type)) {
+					while (current != IntPtr.Zero) {
+						var temp = current;
+						current = Next (temp);
+						g_object_unref (GetData (temp));
+					}
+				} else if (typeof (GLib.Opaque).IsAssignableFrom (element_type)) {
+					while (current != IntPtr.Zero) {
+						var temp = current;
+						current = Next (temp);
+						GLib.Opaque.GetOpaque (GetData (temp), element_type, true).Dispose ();
+					}
+				} else {
+					while (current != IntPtr.Zero) {
+						var temp = current;
+						current = Next (temp);
+						free_func (GetData (temp));
+					}
+				}
+			}
 
 			if (managed)
 				FreeList ();
 		}
 
+		IntPtr GetData (IntPtr current)
+		{
+			// data field is at offset 0 for GList and GSList
+			return Marshal.ReadIntPtr (current);
+		}
+
+		IntPtr Next (IntPtr current)
+		{
+			// next field follows gpointer data field for GList and GSList
+			return Marshal.ReadIntPtr (current, IntPtr.Size);
+		}
+
 		private class ListEnumerator : IEnumerator
 		{
 			private IntPtr current = IntPtr.Zero;
@@ -232,7 +269,7 @@ namespace GLib {
 				current = IntPtr.Zero;
 			}
 		}
-		
+
 		// IEnumerable
 		public IEnumerator GetEnumerator ()
 		{
@@ -250,13 +287,12 @@ namespace GLib {
 		{
 			Empty ();
 		}
-		
+
 		void FreeList ()
 		{
 			if (list_ptr != IntPtr.Zero)
 				Free (list_ptr);
 			list_ptr = IntPtr.Zero;
-			length = -1;
 		}
 
 		// ICloneable
