@@ -28,8 +28,8 @@ namespace GLib {
 	public class Marshaller {
 
 		private Marshaller () {}
-		
-		[DllImport("libglib-2.0-0.dll")]
+
+		[DllImport("libglib-2.0-0.dll", CallingConvention=CallingConvention.Cdecl)]
 		static extern void g_free (IntPtr mem);
 
 		public static void Free (IntPtr ptr)
@@ -46,15 +46,24 @@ namespace GLib {
 				g_free (ptrs [i]);
 		}
 
-		[DllImport("libglib-2.0-0.dll")]
+		[DllImport("libglib-2.0-0.dll", CallingConvention=CallingConvention.Cdecl)]
 		static extern IntPtr g_filename_to_utf8 (IntPtr mem, int len, IntPtr read, out IntPtr written, out IntPtr error);
 
+		[DllImport("libglib-2.0-0.dll", CallingConvention=CallingConvention.Cdecl)]
+		static extern IntPtr g_filename_to_utf8_utf8 (IntPtr mem, int len, IntPtr read, out IntPtr written, out IntPtr error);
+
 		public static string FilenamePtrToString (IntPtr ptr) 
 		{
 			if (ptr == IntPtr.Zero) return null;
 			
 			IntPtr dummy, error;
-			IntPtr utf8 = g_filename_to_utf8 (ptr, -1, IntPtr.Zero, out dummy, out error);
+			IntPtr utf8;
+
+			if (Global.IsWindowsPlatform)
+				utf8 = g_filename_to_utf8_utf8 (ptr, -1, IntPtr.Zero, out dummy, out error);
+			else
+				utf8 = g_filename_to_utf8 (ptr, -1, IntPtr.Zero, out dummy, out error);
+
 			if (error != IntPtr.Zero)
 				throw new GLib.GException (error);
 			return Utf8PtrToString (utf8);
@@ -67,18 +76,30 @@ namespace GLib {
 			return ret;
 		}
 
-		[DllImport("glibsharpglue-2")]
-		static extern UIntPtr glibsharp_strlen (IntPtr mem);
+		[DllImport ("libglib-2.0-0.dll", CallingConvention = CallingConvention.Cdecl)]
+		unsafe static extern char* g_utf8_to_utf16 (IntPtr native_str, IntPtr len, IntPtr items_read, ref IntPtr items_written, out IntPtr error);
 
+		[DllImport ("libglib-2.0-0.dll", CallingConvention = CallingConvention.Cdecl)]
+		unsafe static extern IntPtr g_utf16_to_utf8 (char* native_str, IntPtr len, IntPtr items_read, IntPtr items_written, out IntPtr error);
+
+		[DllImport("glibsharpglue-2", CallingConvention=CallingConvention.Cdecl)]
+		static extern UIntPtr glibsharp_strlen (IntPtr mem);
 		public static string Utf8PtrToString (IntPtr ptr) 
 		{
 			if (ptr == IntPtr.Zero)
 				return null;
-
-			int len = (int) (uint)glibsharp_strlen (ptr);
-			byte[] bytes = new byte [len];
-			Marshal.Copy (ptr, bytes, 0, len);
-			return System.Text.Encoding.UTF8.GetString (bytes);
+			unsafe
+			{
+				IntPtr written = IntPtr.Zero;
+				IntPtr error;
+				char *utf16 = g_utf8_to_utf16 (ptr, new IntPtr (-1), IntPtr.Zero, ref written, out error);
+				if (error != IntPtr.Zero)
+					throw new GLib.GException (error);
+				
+				var result = new string (utf16, 0, (int)written);
+				g_free ((IntPtr)utf16);
+				return result;
+			}
 		}
 
 		public static string[] Utf8PtrToString (IntPtr[] ptrs) {
@@ -106,9 +127,12 @@ namespace GLib {
 			return ret;
 		}
 
-		[DllImport("libglib-2.0-0.dll")]
+		[DllImport("libglib-2.0-0.dll", CallingConvention=CallingConvention.Cdecl)]
 		static extern IntPtr g_filename_from_utf8 (IntPtr mem, int len, IntPtr read, out IntPtr written, out IntPtr error);
 
+		[DllImport("libglib-2.0-0.dll", CallingConvention=CallingConvention.Cdecl)]
+		static extern IntPtr g_filename_from_utf8_utf8 (IntPtr mem, int len, IntPtr read, out IntPtr written, out IntPtr error);
+
 		public static IntPtr StringToFilenamePtr (string str) 
 		{
 			if (str == null)
@@ -116,7 +140,14 @@ namespace GLib {
 
 			IntPtr dummy, error;
 			IntPtr utf8 = StringToPtrGStrdup (str);
-			IntPtr result = g_filename_from_utf8 (utf8, -1, IntPtr.Zero, out dummy, out error);
+
+			IntPtr result;
+
+			if (Global.IsWindowsPlatform)
+				result = g_filename_from_utf8_utf8 (utf8, -1, IntPtr.Zero, out dummy, out error);
+			else
+				result = g_filename_from_utf8 (utf8, -1, IntPtr.Zero, out dummy, out error);
+
 			g_free (utf8);
 			if (error != IntPtr.Zero)
 				throw new GException (error);
@@ -127,11 +158,19 @@ namespace GLib {
 		public static IntPtr StringToPtrGStrdup (string str) {
 			if (str == null)
 				return IntPtr.Zero;
-			byte[] bytes = System.Text.Encoding.UTF8.GetBytes (str);
-			IntPtr result = g_malloc (new UIntPtr ((ulong)bytes.Length + 1));
-			Marshal.Copy (bytes, 0, result, bytes.Length);
-			Marshal.WriteByte (result, bytes.Length, 0);
-			return result;
+
+			unsafe
+			{
+				fixed (char* p = str) {
+					IntPtr error;
+					var result = g_utf16_to_utf8 (p, new IntPtr (str.Length), IntPtr.Zero, IntPtr.Zero, out error);
+
+					if (error != IntPtr.Zero)
+						throw new GLib.GException (error);
+					
+					return result;
+				}
+			}
 		}
 
 		public static string StringFormat (string format, params object[] args) {
@@ -153,7 +192,7 @@ namespace GLib {
 			return result;
 		}
 
-		[DllImport("libglib-2.0-0.dll")]
+		[DllImport("libglib-2.0-0.dll", CallingConvention=CallingConvention.Cdecl)]
 		static extern void g_strfreev (IntPtr mem);
 
 		public static void StrFreeV (IntPtr null_term_array)
@@ -167,7 +206,7 @@ namespace GLib {
 				return new string [0];
 
 			int count = 0;
-			System.Collections.ArrayList result = new System.Collections.ArrayList ();
+			var result = new System.Collections.Generic.List<string> ();
 			IntPtr s = Marshal.ReadIntPtr (null_term_array, count++ * IntPtr.Size);
 			while (s != IntPtr.Zero) {
 				result.Add (Utf8PtrToString (s));
@@ -177,7 +216,7 @@ namespace GLib {
 			if (owned)
 				g_strfreev (null_term_array);
 
-			return (string[]) result.ToArray (typeof(string));
+			return result.ToArray ();
 		}
 
 		public static string[] PtrToStringArrayGFree (IntPtr string_array)
@@ -207,7 +246,7 @@ namespace GLib {
 		// transparently, since we need to alloc buffers of
 		// [native pointer size] * [count] bytes.
 
-		[DllImport("libglib-2.0-0.dll")]
+		[DllImport("libglib-2.0-0.dll", CallingConvention=CallingConvention.Cdecl)]
 		static extern IntPtr g_malloc(UIntPtr size);
 
 		public static IntPtr Malloc (ulong size)
@@ -216,8 +255,8 @@ namespace GLib {
 		}
 
 		static bool check_sixtyfour () {
-			int szint = Marshal.SizeOf (typeof (int));
-			int szlong = Marshal.SizeOf (typeof (long));
+			int szint = sizeof (int);
+			int szlong = sizeof (long);
 			int szptr = IntPtr.Size;
 
 			if (szptr == szint)
@@ -235,7 +274,7 @@ namespace GLib {
 			for (int i = 0; i < args.Length; i++)
 				ptrs[i] = (int) Marshal.StringToHGlobalAuto (args[i]);
 
-			IntPtr buf = g_malloc (new UIntPtr ((ulong) Marshal.SizeOf(typeof(int)) * 
+			IntPtr buf = g_malloc (new UIntPtr ((ulong) sizeof (int) * 
 					       (ulong) args.Length));
 			Marshal.Copy (ptrs, 0, buf, ptrs.Length);
 			return buf;
@@ -248,7 +287,7 @@ namespace GLib {
 			for (int i = 0; i < args.Length; i++)
 				ptrs[i] = (long) Marshal.StringToHGlobalAuto (args[i]);
 				
-			IntPtr buf = g_malloc (new UIntPtr ((ulong) Marshal.SizeOf(typeof(long)) * 
+			IntPtr buf = g_malloc (new UIntPtr ((ulong) sizeof (long) * 
 					       (ulong) args.Length));
 			Marshal.Copy (ptrs, 0, buf, ptrs.Length);
 			return buf;
@@ -320,7 +359,7 @@ namespace GLib {
 			return local_epoch.AddSeconds (time_t.ToInt64 () + utc_offset);
 		}
 
-		[DllImport("glibsharpglue-2")]
+		[DllImport("glibsharpglue-2", CallingConvention=CallingConvention.Cdecl)]
 		static extern IntPtr gtksharp_unichar_to_utf8_string (uint c);
 
 		public static char GUnicharToChar (uint ucs4_char)
@@ -336,7 +375,7 @@ namespace GLib {
 			return ret [0];
 		}
 
-		[DllImport("glibsharpglue-2")]
+		[DllImport("glibsharpglue-2", CallingConvention=CallingConvention.Cdecl)]
 		static extern uint glibsharp_utf16_to_unichar (ushort c);
 
 		public static uint CharToGUnichar (char c)
@@ -351,6 +390,14 @@ namespace GLib {
 			return result;
 		}
 
+		public static IntPtr StructureToPtrAlloc<T> (T o)
+		{
+			IntPtr result = Marshal.AllocHGlobal (Marshal.SizeOf (o));
+			Marshal.StructureToPtr (o, result, false);
+			return result;
+		}
+
+		[Obsolete ("Use the ListPtrToArray<T> overload.")]
 		public static Array ListPtrToArray (IntPtr list_ptr, Type list_type, bool owned, bool elements_owned, Type elem_type)
 		{
 			ListBase list;
@@ -363,6 +410,31 @@ namespace GLib {
 				return ListToArray (list, elem_type);
 		}
 
+		public static T [] ListPtrToArray<T> (IntPtr list_ptr, Type list_type, bool owned, bool elements_owned, ListElementFree free_func)
+		{
+			ListBase list;
+			if (list_type == typeof (GLib.List))
+				list = new GLib.List (list_ptr, typeof (T), owned, elements_owned, free_func);
+			else
+				list = new GLib.SList (list_ptr, typeof (T), owned, elements_owned, free_func);
+
+			using (list)
+				return ListToArray<T> (list);
+		}
+
+		public static T [] ListPtrToArray<T> (IntPtr list_ptr, Type list_type, bool owned, bool elements_owned)
+		{
+			ListBase list;
+			if (list_type == typeof (GLib.List))
+				list = new GLib.List (list_ptr, typeof(T), owned, elements_owned);
+			else
+				list = new GLib.SList (list_ptr, typeof (T), owned, elements_owned);
+
+			using (list)
+				return ListToArray<T> (list);
+		}
+
+		[Obsolete ("Use the PtrArrayToArray<T> overload.")]
 		public static Array PtrArrayToArray (IntPtr list_ptr, bool owned, bool elements_owned, Type elem_type)
 		{
 			GLib.PtrArray array = new GLib.PtrArray (list_ptr, elem_type, owned, elements_owned);
@@ -372,17 +444,52 @@ namespace GLib {
 			return ret;
 		}
 
+		public static T [] PtrArrayToArray<T> (IntPtr list_ptr, bool owned, bool elements_owned)
+		{
+			var elem_type = typeof (T);
+			GLib.PtrArray array = new GLib.PtrArray (list_ptr, elem_type, owned, elements_owned);
+			T [] ret = new T [array.Count];
+			array.CopyTo (ret, 0);
+			array.Dispose ();
+			return ret;
+		}
+
+		public static T [] PtrArrayToArray<T> (IntPtr list_ptr, bool owned, bool elements_owned, ListElementFree free_func)
+		{
+			var elem_type = typeof (T);
+			GLib.PtrArray array = new GLib.PtrArray (list_ptr, elem_type, owned, elements_owned, free_func);
+			T [] ret = new T [array.Count];
+			array.CopyTo (ret, 0);
+			array.Dispose ();
+			return ret;
+		}
+
+		[Obsolete ("Use the ListToArray<T> overload")]
 		public static Array ListToArray (ListBase list, System.Type type)
 		{
-			Array result = Array.CreateInstance (type, list.Count);
-			if (list.Count > 0)
-				list.CopyTo (result, 0);
+			int count = list.Count;
+			Array result = Array.CreateInstance (type, count);
+			if (count > 0)
+				list.CopyTo (result, 0, count);
 
 			if (type.IsSubclassOf (typeof (GLib.Opaque)))
 				list.elements_owned = false;
 
 			return result;
 		}
+
+		public static T [] ListToArray<T> (ListBase list)
+		{
+			int count = list.Count;
+			var result = new T [count];
+			if (count > 0)
+				list.CopyTo (result, 0);
+
+			if (typeof(T).IsSubclassOf (typeof (GLib.Opaque)))
+				list.elements_owned = false;
+
+			return result;
+		}
 	}
 }
 
