@@ -26,6 +26,7 @@ namespace GLib {
 	using System.Collections;
 	using System.Collections.Generic;
 	using System.ComponentModel;
+	using System.Diagnostics;
 	using System.Reflection;
 	using System.Runtime.InteropServices;
 	using System.Text;
@@ -34,8 +35,7 @@ namespace GLib {
 
 		IntPtr handle;
 		ToggleRef tref;
-		bool disposed = false;
-		bool owned = true;
+		internal protected bool owned = false;
 		Hashtable data;
 		static Dictionary<IntPtr, ToggleRef> Objects = new Dictionary<IntPtr, ToggleRef>(IntPtrEqualityComparer.Instance);
 		static object lockObject = new object ();
@@ -46,13 +46,12 @@ namespace GLib {
 		{
 			lock (lockObject) {
 				lock (Objects) {
-					ToggleRef res;
-					if (Objects.TryGetValue (handle, out res))
-						PendingDestroys.Add (res);
+					if (tref != null)
+						PendingDestroys.Add (tref);
 					Objects.Remove (Handle);
 				}
 				if (!idle_queued){
-					Timeout.Add (50, new TimeoutHandler (PerformQueuedUnrefs));
+					Timeout.Add (50, PerformQueuedUnrefs);
 					idle_queued = true;
 				}
 			}
@@ -79,21 +78,20 @@ namespace GLib {
 
 		public virtual void Dispose ()
 		{
-			if (disposed)
+			if (Handle == IntPtr.Zero)
 				return;
-
-			disposed = true;
-			ToggleRef toggle_ref;
+			
 			lock(Objects) {
-				Objects.TryGetValue (Handle, out toggle_ref);
 				Objects.Remove (Handle);
-			}
-			try {
-				if (toggle_ref != null)
-					toggle_ref.Free ();
-			} catch (Exception e) {
-				Console.WriteLine ("Exception while disposing a " + this + " in Gtk#");
-				throw e;
+				try {
+					if (tref != null) {
+						tref.Free ();
+						tref = null;
+					}
+				} catch (Exception e) {
+					Console.WriteLine ("Exception while disposing a " + this + " in Gtk#");
+					throw e;
+				}
 			}
 			handle = IntPtr.Zero;
 			GC.SuppressFinalize (this);
@@ -250,7 +248,7 @@ namespace GLib {
 		static void GetPropertyCallback (IntPtr handle, uint property_id, ref GLib.Value value, IntPtr param_spec)
 		{
 			GLib.Object obj = GLib.Object.GetObject (handle, false);
-			value.Val = (Properties [param_spec] as PropertyInfo).GetValue (obj, new object [0]);
+			value.Val = (Properties [param_spec] as PropertyInfo).GetValue (obj);
 		}
 
 		static GetPropertyDelegate get_property_handler;
@@ -268,7 +266,7 @@ namespace GLib {
 		static void SetPropertyCallback(IntPtr handle, uint property_id, ref GLib.Value value, IntPtr param_spec)
 		{
 			GLib.Object obj = GLib.Object.GetObject (handle, false);
-			(Properties [param_spec] as PropertyInfo).SetValue (obj, value.Val, new object [0]);
+			(Properties [param_spec] as PropertyInfo).SetValue (obj, value.Val);
 		}
 
 		static SetPropertyDelegate set_property_handler;
@@ -409,6 +407,7 @@ namespace GLib {
 
 		protected void CreateNativeObject (IntPtr [] native_names, GLib.Value [] vals, int count)
 		{
+			owned = true;
 			Raw = gtksharp_object_newv (LookupGType ().Val, count, native_names, vals);
 			for (int i = 0; i < count; ++i) {
 				GLib.Marshaller.Free (native_names [i]);
@@ -474,6 +473,7 @@ namespace GLib {
 			}
 		}
 
+		[DebuggerBrowsable (DebuggerBrowsableState.Never)]
 		public IntPtr OwnedHandle {
 			get {
 				return g_object_ref (handle);
