@@ -28,68 +28,30 @@ namespace GLib {
 
 	public class Timeout {
 
-		[UnmanagedFunctionPointer (CallingConvention.Cdecl)]
-		delegate bool TimeoutHandlerInternal ();
-
 		internal class TimeoutProxy : SourceProxy {
+			readonly TimeoutHandler real_handler;
 			public TimeoutProxy (TimeoutHandler real)
 			{
 				real_handler = real;
-				proxy_handler = new TimeoutHandlerInternal (Handler);
 			}
 
-			~TimeoutProxy ()
+			protected override bool Invoke (IntPtr data)
 			{
-				Dispose (false);
-			}
-
-			public void Dispose ()
-			{
-				Dispose (true);
-				GC.SuppressFinalize (this);
-			}
-
-			protected virtual void Dispose (bool disposing)
-			{
-				// Both branches remove our delegate from the
-				// managed list of handlers, but only
-				// Source.Remove will remove it from the
-				// unmanaged list also.
-
-				if (disposing)
-					Remove ();
-				else
-					Source.Remove (ID);
-			}
-
-			public bool Handler ()
-			{
-				try {
-					TimeoutHandler timeout_handler = (TimeoutHandler) real_handler;
-
-					bool cont = timeout_handler ();
-					if (!cont)
-						Remove ();
-					return cont;
-				} catch (Exception e) {
-					ExceptionManager.RaiseUnhandledException (e, false);
-				}
-				return false;
+				return real_handler ();
 			}
 		}
 
 		private Timeout () {}
 
 		[DllImport("libglib-2.0-0.dll", CallingConvention=CallingConvention.Cdecl)]
-		static extern uint g_timeout_add (uint interval, TimeoutHandlerInternal d, IntPtr data);
+		static extern uint g_timeout_add (uint interval, SourceProxy.GSourceFuncInternal d, IntPtr data);
 
 		public static uint Add (uint interval, TimeoutHandler hndlr)
 		{
 			TimeoutProxy p = new TimeoutProxy (hndlr);
 
-			p.ID = g_timeout_add (interval, (TimeoutHandlerInternal) p.proxy_handler, IntPtr.Zero);
-			lock (Source.source_handlers)
-				Source.source_handlers [p.ID] = p;
+			p.ID = g_timeout_add (interval, p.Handler, IntPtr.Zero);
+			Source.Add (p);
 
 			return p.ID;
 		}
