@@ -212,7 +212,8 @@ namespace GLib {
 
 		public void Dispose () 
 		{
-			g_value_unset (ref this);
+			if (type != IntPtr.Zero)
+				g_value_unset (ref this);
 		}
 
 		public void Init (GLib.GType gtype)
@@ -344,11 +345,7 @@ namespace GLib {
 			else if (t.IsSubclassOf (typeof (GLib.Opaque)))
 				return (GLib.Opaque) this;
 
-			MethodInfo mi = t.GetMethod ("New", BindingFlags.Static | BindingFlags.Public | BindingFlags.FlattenHierarchy);
-			if (mi == null)
-				return Marshal.PtrToStructure (boxed_ptr, t);
-			else
-				return mi.Invoke (null, new object[] {boxed_ptr});
+			return FastActivator.CreateBoxed (boxed_ptr, t);
 		}
 
 		public object Val
@@ -428,7 +425,8 @@ namespace GLib {
 						return;
 					}
 					IntPtr wrapper = ManagedValue.WrapObject (value);
-					g_value_unset (ref this);
+					if (type != IntPtr.Zero)
+						g_value_unset (ref this);
 					g_value_init (ref this, ManagedValue.GType.Val);
 					g_value_set_boxed (ref this, wrapper);
 					ManagedValue.ReleaseWrapper (wrapper);
