@@ -23,6 +23,21 @@
 
 #include <gtk/gtkcellrenderer.h>
 
+static const gchar *__prefix = "__gtksharp_";
+
+#define HAS_PREFIX(a) (*((guint64 *)(a)) == *((guint64 *) __prefix))
+
+static GObjectClass *
+get_threshold_class (GObject *obj)
+{
+	GType gtype = G_TYPE_FROM_INSTANCE (obj);
+	while (HAS_PREFIX (g_type_name (gtype)))
+		gtype = g_type_parent (gtype);
+	GObjectClass *klass = g_type_class_peek (gtype);
+	if (klass == NULL) klass = g_type_class_ref (gtype);
+	return klass;
+}
+
 void gtksharp_cellrenderer_invoke_get_size (GType gtype, GtkCellRenderer *cell, GtkWidget *widget, GdkRectangle *cell_area, gint *x_offset, gint *y_offset, gint *width, gint *height);
 
 void
@@ -37,7 +52,7 @@ void gtksharp_cellrenderer_base_get_size (GtkCellRenderer *cell, GtkWidget *widg
 void
 gtksharp_cellrenderer_base_get_size (GtkCellRenderer *cell, GtkWidget *widget, GdkRectangle *cell_area, gint *x_offset, gint *y_offset, gint *width, gint *height)
 {
-	GtkCellRendererClass *parent = g_type_class_peek_parent (G_OBJECT_GET_CLASS (cell));
+	GtkCellRendererClass *parent = (GtkCellRendererClass *)get_threshold_class (G_OBJECT (cell));
 	if (parent->get_size)
 		(*parent->get_size) (cell, widget, cell_area, x_offset, y_offset, width, height);
 }
@@ -67,7 +82,7 @@ void gtksharp_cellrenderer_base_render (GtkCellRenderer *cell, GdkDrawable *wind
 void
 gtksharp_cellrenderer_base_render (GtkCellRenderer *cell, GdkDrawable *window, GtkWidget *widget, GdkRectangle *background_area, GdkRectangle *cell_area, GdkRectangle *expose_area, GtkCellRendererState flags)
 {
-	GtkCellRendererClass *parent = g_type_class_peek_parent (G_OBJECT_GET_CLASS (cell));
+	GtkCellRendererClass *parent = (GtkCellRendererClass *)get_threshold_class (G_OBJECT (cell));
 	if (parent->render)
 		(*parent->render) (cell, window, widget, background_area, cell_area, expose_area, flags);
 }
@@ -97,7 +112,7 @@ GtkCellEditable * gtksharp_cellrenderer_base_start_editing (GtkCellRenderer *cel
 GtkCellEditable *
 gtksharp_cellrenderer_base_start_editing (GtkCellRenderer *cell, GdkEvent *event, GtkWidget *widget, const gchar *path, GdkRectangle *background_area, GdkRectangle *cell_area, GtkCellRendererState flags)
 {
-	GtkCellRendererClass *parent = g_type_class_peek_parent (G_OBJECT_GET_CLASS (cell));
+	GtkCellRendererClass *parent = (GtkCellRendererClass *)get_threshold_class (G_OBJECT (cell));
         if (parent->start_editing)
 		return (*parent->start_editing) (cell, event, widget, path, background_area, cell_area, flags);
 	return NULL;
