@@ -23,14 +23,14 @@ namespace Pango {
 
 	public class AttrGravity : Attribute {
 
-		[DllImport("libpango-1.0-0.dll")]
+		[DllImport("libpango-1.0-0.dll", CallingConvention=CallingConvention.Cdecl)]
 		static extern IntPtr pango_attr_gravity_new (int gravity);
 
-		public AttrGravity (Gravity gravity) : this (pango_attr_gravity_new ((int) gravity)) {}
+		public AttrGravity (Gravity gravity) : this (pango_attr_gravity_new ((int) gravity), true) {}
 
-		internal AttrGravity (IntPtr raw) : base (raw) {}
+		internal AttrGravity (IntPtr raw, bool owned) : base (raw, owned) {}
 
-		[DllImport("pangosharpglue-2")]
+		[DllImport("pangosharpglue-2", CallingConvention=CallingConvention.Cdecl)]
 		static extern int pangosharp_attr_int_get_value (IntPtr raw);
 
 		public Gravity Gravity {
