@@ -26,9 +26,9 @@ namespace Pango {
 		[DllImport("libpango-1.0-0.dll", CallingConvention=CallingConvention.Cdecl)]
 		static extern IntPtr pango_attr_rise_new (int rise);
 
-		public AttrRise (int rise) : this (pango_attr_rise_new (rise)) {}
+		public AttrRise (int rise) : this (pango_attr_rise_new (rise), true) {}
 
-		internal AttrRise (IntPtr raw) : base (raw) {}
+		internal AttrRise (IntPtr raw, bool owned) : base (raw, owned) {}
 
 		[DllImport("pangosharpglue-2", CallingConvention=CallingConvention.Cdecl)]
 		static extern int pangosharp_attr_int_get_value (IntPtr raw);
