@@ -23,14 +23,14 @@ namespace Pango {
 
 	public class AttrWeight : Attribute {
 
-		[DllImport("libpango-1.0-0.dll")]
+		[DllImport("libpango-1.0-0.dll", CallingConvention=CallingConvention.Cdecl)]
 		static extern IntPtr pango_attr_weight_new (Pango.Weight weight);
 
-		public AttrWeight (Pango.Weight weight) : this (pango_attr_weight_new (weight)) {}
+		public AttrWeight (Pango.Weight weight) : this (pango_attr_weight_new (weight), true) {}
 
-		internal AttrWeight (IntPtr raw) : base (raw) {}
+		internal AttrWeight (IntPtr raw, bool owned) : base (raw, owned) {}
 
-		[DllImport("pangosharpglue-2")]
+		[DllImport("pangosharpglue-2", CallingConvention=CallingConvention.Cdecl)]
 		static extern int pangosharp_attr_int_get_value (IntPtr raw);
 
 		public Pango.Weight Weight {
